/**************************************************************/
/*                                                            */
/*   File name :  liblsq.c                                    */
/*                                                            */
/*      Version:   2005-05-12                                 */
/*                                                            */
/*   Copyright (c) 2005 T.Kondo/NICT(CRL) All Right Reserved  */
/*                                                            */
/**************************************************************/
/*

Ver: 2005-05-12
     Start migration from Fortran liblsq library

*/
#include "liblsq.h"


void lsqinit(lsq_type_t *lt, int maxobs, int maxpara)
{
    int n;

    lt->m_omax=maxobs;
    lt->m_pmax=maxpara;

    /* memory allocation */
    lt->r_y=(double *)malloc(lt->m_omax*sizeof(double));
    lt->r_yerr=(double *)malloc(lt->m_omax*sizeof(double));
    lt->i_good=(int *)malloc(lt->m_omax*sizeof(int));
    lt->d_a=(double *)malloc(lt->m_omax*lt->m_pmax*sizeof(double));
    lt->r_omc=(double *)malloc(lt->m_omax*sizeof(double));
    lt->x_adj=(double *)malloc(lt->m_pmax*sizeof(double));
    lt->i_flag=(int *)malloc(lt->m_pmax*sizeof(int));
    lt->d_corx=(double *)malloc(lt->m_pmax*lt->m_pmax*sizeof(double));
    lt->d_covx=(double *)malloc(lt->m_pmax*lt->m_pmax*sizeof(double));

    /*.... All Obs flag reset ....*/
    for(n=0; n<lt->m_omax; n++) lt->i_good[n]=0;
    /*.... All parameter flag set ..*/
    for(n=0; n<lt->m_pmax; n++) lt->i_flag[n]=1;

    lt->i_dbk=0;
}

void lsqfinish(lsq_type_t *lt)
{
    /* memory release */
    free(lt->r_y);
    free(lt->r_yerr);
    free(lt->i_good);
    free(lt->d_a);
    free(lt->r_omc);
    free(lt->x_adj);
    free(lt->i_flag);
    free(lt->d_corx);
    free(lt->d_covx);
}

void lsqdebug(lsq_type_t *lt)
{
    lt->i_dbk=1;
}


void lsqgetmax(lsq_type_t *lt, int *maxobs, int *maxpara)
{
    *maxobs=lt->m_omax;
    *maxpara=lt->m_pmax;
}


void lsqsetsize(lsq_type_t *lt, int nobs, int npara)
{
    lt->n_data=nobs;
    lt->n_para=npara;
}

void lsqgetsize(lsq_type_t *lt, int *nobs, int *npara)
{
    *nobs=lt->n_data;
    *npara=lt->n_para;
}


void lsqsetflag(lsq_type_t *lt, char *a1, int nm, char *a2)
{
    if(strncmp(a1,"PAR",3)==0){
        if(strncmp(a2,"YES",3)==0){
            lt->i_flag[nm-1]=1;
        } else if(strncmp(a2,"NO",2)==0){
            lt->i_flag[nm-1]=0;
        }
    } else if(strncmp(a1,"OBS",3)==0){
        if(strncmp(a2,"GOOD",4)==0){
            lt->i_good[nm-1]=1;
        } else if(strncmp(a2,"BAD",3)==0){
            lt->i_good[nm-1]=0;
        }
    }
}

void lsqgetflag(lsq_type_t *lt, char *a1, int nm, int *iflag)
{
    if(strncmp(a1,"PAR",3)==0){
        *iflag=lt->i_flag[nm-1];
    } else if(strncmp(a1,"OBS",3)==0){
        *iflag=lt->i_good[nm-1];
    }
}


void lsqpflag(lsq_type_t *lt, int m)
{
    lt->i_flag[m-1]=1;
}

void lsqoflag(lsq_type_t *lt, int n)
{
    lt->i_good[n-1]=1;
}


void lsqpflagr(lsq_type_t *lt, int m)
{
    lt->i_flag[m-1]=10;
}

void lsqoflagr(lsq_type_t *lt, int n)
{
    lt->i_good[n-1]=0;
}

void lsqgetpflag(lsq_type_t *lt, int m, int *iflag)
{
    *iflag=lt->i_flag[m-1];
}

void lsqgetoflag(lsq_type_t *lt, int n, int *igood)
{
    *igood=lt->i_good[n-1];
}

void lsqsetjacob(lsq_type_t *lt, double a, int n, int m)
{
    lt->ierr=0;
    if(n > lt->m_omax){
        lt->ierr=-1;
        printf(" Data position (Obs direction) exceeds limit (%d)\n",lt->m_omax);
        goto exit2return;
    }
    if(m > lt->m_pmax){
        lt->ierr=-2;
        printf(" Data position (Para direction) exceeds limit (%d)\n",lt->m_pmax);
        goto exit2return;
    }
    /*  D_A(N,M)=A */
    lt->d_a[n-1 + lt->m_omax*(m-1)]=a;
exit2return:;
}

void lsqgetjacob(lsq_type_t *lt, double *a, int n, int m)
{
    lt->ierr=0;
    if(n > lt->m_omax){
        lt->ierr=-1;
        printf(" Data position (Obs direction) exceeds limit (%d)\n",lt->m_omax);
        goto exit2return;
    }
    if(m > lt->m_pmax){
        lt->ierr=-2;
        printf(" Data position (Para direction) exceeds limit (%d)\n",lt->m_pmax);
        goto exit2return;
    }
    /*  A=D_A(N,M) */
    *a=lt->d_a[n-1 + lt->m_omax*(m-1)];
exit2return:;
}


void lsqsetobs(lsq_type_t *lt, double y, double yerr, int n)
{
    if(n > lt->m_omax){
        printf(" Data position (Obs direction) exceeds limit (%d)\n",lt->m_omax);
        goto exit2return;
    }
    lt->r_y[n-1]=y;
    lt->r_yerr[n-1]=yerr;
    lt->i_good[n-1]=1;
exit2return:;
}

void lsqgetobs(lsq_type_t *lt, double *y, double *yerr, int n)
{
    if(n > lt->m_omax){
        printf(" Data position (Obs direction) exceeds limit (%d)\n",lt->m_omax);
        goto exit2return;
    }
    *y=lt->r_y[n-1];
    *yerr=lt->r_yerr[n-1];
exit2return:;
}

void lsqgetkai(lsq_type_t *lt, double *x, double *xerr, int m)
{
    if(m > lt->n_para) goto exit2return;
    /*  X=X_ADJ(M)
      XERR=DSQRT(R_CHI2*DABS(D_COVX(M,M))) */
    *x=lt->x_adj[m-1];
    *xerr=sqrt(lt->r_chi2*fabs(lt->d_covx[m-1+lt->m_pmax*(m-1)]));
exit2return:;
}

void lsqgetcor(lsq_type_t *lt, double *c, int m1, int m2)
{
    if(m1 > lt->n_para || m2 > lt->n_para ) goto exit2return;
    /*  C=D_CORX(M1,M2) */
    *c=lt->d_corx[m1-1+lt->m_pmax*(m2-1)];
exit2return:;
}

void lsqgetcov(lsq_type_t *lt, double *v, int m1, int m2)
{
    if(m1 > lt->n_para || m2 > lt->n_para ) goto exit2return;
    /*  C=D_CORX(M1,M2) */
    *v=lt->d_covx[m1-1+lt->m_pmax*(m2-1)];
exit2return:;
}

void lsqgetstd(lsq_type_t *lt, double *rsum2, double *sigma,
               double *rchi2, int *nfree, int *ndata, int *mpara)
{
    *rsum2=lt->r_sum2;
    *sigma=lt->r_sigma;
    *rchi2=lt->r_chi2;
    *nfree=lt->n_free;
    *ndata=lt->n_obs;
    *mpara=lt->m_para;
}

void lsqgetomc(lsq_type_t *lt, double *omc, int n)
{
    *omc=lt->r_omc[n-1];
}

void dolsq(lsq_type_t *lt)
{
/*
C
C
      SUBROUTINE DOLSQ(IERR)
********************************************************************
*    LEAST SQARES METHOD BY S.V.D                                  *
*             by T.Kon for general purpose                         *
*          91-05-10                                                *
*                                                                  *
*      Weighting with unit (i.e.  W=1/ERR**2) is adopted here      *
*      so as to handle different unit obs data in the fitting      *
*                                                                  *
*     Maximum # of parameters  100                                 *
*     Maximum # of data       1500                                 *
*                                                                  *
*                                                                  *
*     I/O   via EMA array                                          *
*                                                                  *
*     lt.IERR= 0   :  normal return                                *
*     lt.IERR=-1   :  Error occured                                *
********************************************************************
*/
/*
C
      IMPLICIT REAL*8(D)
C
      DIMENSION
     . IAJST(100)    ! Adjust parameter indicator from origin to reduced size
C
      INCLUDE "EMA_IN_LSQ.TXT"
CC    COMMON /LSQ/
C       .......  Data Prepared by main routine and return to main
CC   . R_Y(2500)       ! Observed data (O-C in case of non linear lsq)
CC   .,R_YERR(2500)    ! 1 sigma error of observed data
CC   .,I_GOOD(2500)    ! Flag indicate good obs data   1:OK  0: NO
CC   .,D_A(2500,100)   ! Jacobian (oroginal) not affected
CC   .,R_OMC(2500)     ! O-C after fitting            (return)
CC   .,X_ADJ(100)      ! Adjustment value calculated
CC                     !    must be added to paramters in main
CC   .,I_FLAG(100)     ! Adjusting parameter indicator
CC                     !   1: Adjust  0 : no Adjust  will be affected
CC                     !    -1 means rank ochi occurred parameter
CC   .,D_CORX(100,100) ! Obtained correlation matrix (return)
CC   .,D_COVX(100,100) ! Obtained covariance matrix  (return)
CC   .,R_SUM2          ! Sum of residual square   (return)
CC   .,R_SIGMA         ! Residual 1 sigma         (return)
CC   .,R_CHI2          ! Reduced chi squer        (return)
CC   .,N_FREE          ! Degree of freedum        (return)
CC   .,M_PMAX          ! Available max # of parameters =100
CC   .,M_OMAX          !    "               data =2500
CC   .,N_DATA          ! # of data valid   <= M_OMAX
CC   .,N_PARA          ! # of parameters in X_PRM etc.
CC   .,M_PARA          ! Actual # of parameters fitted    (Return)
CC   .,I_DBK           ! Debug control   1 : DEBUG ON  0 : DEBUG OFF
C
      COMMON /LSQWRK/
C       .......  Arrays used in this routine only
     . D_WA(2500,100)      !  Jacobian weighted and reordered wrt parametrs
     .,DU(2500,100)        !  U of SVD bunkai
     .,DV(2500,100)        !  V of SVD bunkai
     .,DSIGMA(100)         !  Contain singular value
     .,DWORK(100)          !  Work array
     .,D_Y(2500)           !  Weighted Observed data
     .,DFACT(100)          !  Scaling factor
C
      DATA EPSM/2.0E-16/      ! Machine epsiron
*/
    double *d_wa;  /* D_WA(2500,100) Jacobian weighted and reordered wrt parametrs */
    double *du;  /* DU(2500,100)        !  U of SVD bunkai */
    double *dv;  /* DV(2500,100)        !  V of SVD bunkai */
    double *dsigma;  /* DSIGMA(100)         !  Contain singular value */
    double *dwork;  /* DWORK(100)          !  Work array */
    double *d_y;  /* D_Y(2500)      !  Weighted Observed data */
    double *dfact;  /* DFACT(100)   !  Scaling factor */
    int  *iajst;  /* IAJST(100)    ! Adjust parameter indicator from origin to reduced size */

    double damx,dsig1,relerr,dtau,dsmax,dsmin,cond,dsum,dbunbo,rsum,ominusc;
    int idbk,maxx,maxy,nobs,irank;
    int i,j,k,m,n,m1,m2,m1pos,m2pos,nsum;
    int mpos,kerr;
    double epsm=2.0e-16;  /* computer's epsiron */

    idbk=lt->i_dbk;
    maxx=lt->m_pmax;
    maxy=lt->m_omax;
    nobs=lt->n_data;

    d_wa  =(double *)malloc(maxy*maxx*sizeof(double));
    du    =(double *)malloc(maxy*maxx*sizeof(double));
    dv    =(double *)malloc(maxy*maxx*sizeof(double));
    dsigma=(double *)malloc(maxx*sizeof(double));
    dwork =(double *)malloc(maxx*sizeof(double));
    d_y   =(double *)malloc(maxy*sizeof(double));
    dfact =(double *)malloc(maxx*sizeof(double));
    iajst=(int *)malloc(maxx*sizeof(double));
   

/*
C
C     IDBK=1       ! DEBUG ON
C     IDBK=0       ! DEBUG OFF
      IDBK=I_DBK
      LUO=1        ! DEBUG OUT LU
C
C     M_PMAX=100      ! PARAMETER MAX
C     M_OMAX=2500     ! MATRIX ROW MAX (AVAILABLE MAX OBS NUMBER)
C
      MAXX=M_PMAX
      MAXY=M_OMAX
      NOBS=N_DATA
C
C     Now D_A(MAXY,MAXX) is a Jacobian Array
C         D_Y(MAXY)     is a observed data
C
*/
/*
     -----Adjust and skip of parameter order....
C     .... Make conversion table that convert parameter position in
C          this routine to actual parameter position
C
*/

    for(m=0; m<maxx; m++) iajst[m]=0;

    k=0;
    for(m=1; m<=lt->n_para; m++){
        if(lt->i_flag[m-1] != 0){
            k++;
            iajst[k-1]=m;
        }
    }
    lt->m_para=k; /* Actual number of parameters adjusted */
    if(lt->i_dbk == 1){
        printf("dolsq: Flag monitor\n");
        printf("dolsq: Raw Flag ");
        for(m=0; m<lt->n_para; m++) printf(" %d",lt->i_flag[m]);
        printf("\n");
        printf("dolsq: Adjust   ");
        for(m=0; m<lt->m_para; m++) printf(" %d",iajst[m]);
        printf("\n");
    }

    /*...... Jacobian copy to D_WA() with weighting and rearrange */

    for(n=1; n<=nobs; n++){
        /* D_WA clear wrt parameters */
        for(m=1; m<=maxx; m++){
            d_wa[n-1+(m-1)*maxy]=0.0;
        }
        d_y[n-1]=0.0;
        /* 
         ..... WEIGHTING  Jacobian and Obeserved data...
           ....  D_Y vector weighting ...
           ... We adopted weighting with unit so as to allow different
               observed value (in unit) in obs data
          ..... Jacovian reorder and weighting (if so)
        */
        if(lt->i_good[n-1] > 0){
            d_y[n-1]=lt->r_y[n-1]/lt->r_yerr[n-1];
            for(m=1; m<=lt->m_para; m++){
                mpos=iajst[m-1]; /* Actual parameter position in Jacobian */
                d_wa[(n-1)+(m-1)*maxy]=lt->d_a[(n-1)+(mpos-1)*maxy]/lt->r_yerr[n-1];
            }
        }
    }
/*
C     Scaling of Jacobian to improve condition number
C     ..... Scaling factor is taken here as a max of ABS(D_WA) in each
C              parameters.
*/
     
    for(m=1; m<=lt->m_para; m++){
        damx=fabs(d_wa[0+(m-1)*maxy]);
        for(n=1; n<=nobs; n++){
            if(fabs(d_wa[(n-1)+(m-1)*maxy]) > damx) damx=fabs(d_wa[(n-1)+(m-1)*maxy]);
        }
        if(damx < 1.0e-20){
            dfact[m-1]=1.0;
        } else {
            dfact[m-1]=1.0/damx;
        }
    }
    if(lt->i_dbk == 1){
        printf("dolsq: SCALING FACTOR (DFACT) \n");
        for(n=0; n<lt->m_para; n++) printf(" %f",dfact[n]);
        printf("\n");
    }
    /*     ..... do scaling of Jacobian ...... */
    for(m=1; m<=lt->m_para; m++){
        for(n=1; n<=nobs; n++){
            d_wa[(n-1)+(m-1)*maxy]*=dfact[m-1];
        }
    }

    /* ----------- Singular Value Decomposition (SVD) ------ */

    dwsvd(maxy,nobs,maxx,lt->m_para,d_wa,dsigma,1,du,1,dv,&kerr,dwork);

    if(lt->i_dbk == 1){
        printf("dolsq: Singular Value (DSIGMA)\n");
        for(i=0; i<lt->m_para; i++) printf(" %f",dsigma[i]);
        printf("\n");
    }
    lt->ierr=0;
    if(kerr != 0){
        printf("dolsq/ DWSVD_2 ERROR %d\n",kerr);
        lt->ierr=-1;
        return;
    }
    /*......... Maximum singular value finding ............*/
    dsig1=0.0;
    for(m=1; m<=lt->m_para; m++){
        if(dsigma[m-1] > dsig1) dsig1=dsigma[m-1];
    }
    /*........ Absolute error limit definition ............*/

    relerr=(double)(lt->m_para)*epsm;
    dtau=relerr*dsig1;
    /*........ Finding null coefficent */

    irank=lt->m_para;
    dsmax=0.0;
    dsmin=1.0e30;
    for(m=1; m<= lt->m_para; m++){
        if(dsigma[m-1] > dtau){
            if(dsigma[m-1] > dsmax) dsmax=dsigma[m-1];
            if(dsigma[m-1] < dsmin) dsmin=dsigma[m-1];
        } else {   /* rank defficency occurred */
            mpos=iajst[m-1]; /* MPOS is actual parameter position in main */
            lt->i_flag[mpos-1]=-1; /* mean where rank deficiency occurred */
            printf("dolsq/  Singular Value for Parameter # %d is le DTAU(%14.3e)\n",
                mpos,dtau);
            printf("dolsq/  This parameter (%d) is not adjusted here!\n",mpos);
            irank-=1;
            printf("dolsq/  NULL COEFFICIENTS\n");
            for(k=1; k<=lt->m_para; k++) printf(" %f",dv[(k-1)+(m-1)*maxy]);
            printf("\n");
        }
    }
    /*..... CHECK CONDITION NUMBER .....*/
    cond=dsmax/dsmin;

    if(lt->i_dbk==1){
       printf("dolsq: # of Parameters = %d    Rank = %d\n",lt->m_para,irank);
       if(lt->m_para==irank) printf("       Coefficient are unique \n");
       printf("       SVmax= %f   SVmin= %f\n",dsmax,dsmin);
       printf("       CONDITION NUMBER = %f\n",cond);
    }

    if(cond > 1.0/epsm){
       printf("dolsq/  !!!!!!!! WARNING !!!!!!!!!!!!\n");
       printf("dolsq/  !!  BAD CONDITION NUMNER  !!!\n");
       printf("dolsq/  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
    }
    /*........ ADJUSTING VECTOR CALC .....*/

    for(m=0; m<maxx; m++) lt->x_adj[m]=0.0;

    for(m=1; m<=lt->m_para; m++){
        mpos=iajst[m-1];
        if(lt->i_flag[mpos-1] > 0){ /* Adjusting vector calc */
            dsum=0.0;
            for(n=1; n<=nobs; n++){
                dsum+=du[(n-1)+(m-1)*maxy]*d_y[n-1];
            }
            dsum/=dsigma[m-1];
            for(i=1; i<=lt->m_para; i++){
                lt->x_adj[i-1]+=dsum*dv[(i-1)+(m-1)*maxy];
            }
        }
    }
    for(j=0; j<lt->m_para; j++) lt->x_adj[j]*=dfact[j];

    if(lt->i_dbk==1){
       printf("dolsq: ADJUSTING VECTOR X_ADJ(I)\n");
       for(i=0; i<lt->m_para; i++) printf(" %f",lt->x_adj[i]);
       printf("\n");
    }

    /*..... COVARIANCE MATRIX CALC    D_COVX(100,100).........*/

    for(n=1; n<=lt->m_para; n++){
        for(m=1; m<=lt->m_para; m++){
            dsum=0.0;
            for(k=1; k<=lt->m_para; k++){
                mpos=iajst[k-1];
                if(lt->i_flag[mpos-1] > 0){
                    dsum+=dv[(n-1)+(k-1)*maxy]*dv[(m-1)+(k-1)*maxy]/(dsigma[k-1]*dsigma[k-1]);
                }
            }
            lt->d_covx[(n-1)+(m-1)*maxx]=dsum;
        }
    }

    /*..... Do scaling of covariance matrix using DFACT ..*/
    for(n=1; n<=lt->m_para; n++){
        for(m=1; m<=lt->m_para; m++){
            lt->d_covx[(n-1)+(m-1)*maxx]*=(dfact[n-1]*dfact[m-1]);
        }
    }

    if(lt->n_para != lt->m_para){

       /*...... Re arrange acording to actual parameter position
         .... Rearange X_ADJ
          here R_OMC  used for temporal save
       */
        for(m=1; m<=lt->n_para; m++){  /* N_PARA is initial # of parameter */
            lt->r_omc[m-1]=lt->x_adj[m-1];
            lt->x_adj[m-1]=0.0;
        }

        for(m=1; m<=lt->m_para; m++){  /* M_PARA is # of parameters adjusted */
            mpos=iajst[m-1];
            lt->x_adj[mpos-1]=lt->r_omc[m-1];
        }

        /*... Rearange D_COVX     */
        /*..... D_CORX is used as work array here */
        for(m1=1; m1<=lt->n_para; m1++){
            for(m2=1; m2<=lt->n_para; m2++){
                lt->d_corx[(m1-1)+(m2-1)*maxx]=0.0;
            }
        }
        for(m1=1; m1<=lt->m_para; m1++){
            m1pos=iajst[m1-1];
            if(lt->i_flag[m1pos-1] > 0){
                for(m2=1; m2<=lt->m_para; m2++){
                    m2pos=iajst[m2-1];
                    if(lt->i_flag[m2pos-1] > 0){
                        lt->d_corx[(m1pos-1)+(m2pos-1)*maxx]=lt->d_covx[(m1-1)+(m2-1)*maxx];
                    }
                }
            }
        }

        /* ... copy back */
        for(m1=1; m1<=lt->n_para; m1++){
            for(m2=1; m2<=lt->n_para; m2++){
                lt->d_covx[(m1-1)+(m2-1)*maxx]=lt->d_corx[(m1-1)+(m2-1)*maxx];
            }
        }

        /*... Now position of D_COVX correspond actual parameter position */

    }
    /*  End of rearanging */

    if(lt->i_dbk==1){
        printf("dolsq: COVARIANCE MATRIX\n");
        for(m=1; m<=lt->n_para; m++){
            printf("%4d",m);
            for(i=1; i<=lt->n_para; i++) printf(" %f",lt->d_covx[(m-1)+(i-1)*maxx]);
            printf("\n");
        }
    }

    /*...... CORRELATION MATRIX CALC  D_CORX(100,100).........*/

    for(m1=1; m1<=lt->n_para; m1++){
        for(m2=1; m2<=lt->n_para; m2++){
            lt->d_corx[(m1-1)+(m2-1)*maxx]=0.0;
            if(lt->i_flag[m1-1] > 0 && lt->i_flag[m2-1] > 0){
                /* calculation for actual parameter only */
                dbunbo=sqrt(fabs(lt->d_covx[(m1-1)+(m1-1)*maxx]*lt->d_covx[(m2-1)+(m2-1)*maxx]));
                lt->d_corx[(m1-1)+(m2-1)*maxx]=lt->d_covx[(m1-1)+(m2-1)*maxx]/dbunbo;
            }
        }
    }

    if(lt->i_dbk==1){
        printf("dolsq: CORRELATION MATRIX\n");
        for(m=1; m<=lt->n_para; m++){
            printf("%4d",m);
            for(i=1; i<=lt->n_para; i++) printf(" %10.3f",lt->d_corx[(m-1)+(i-1)*maxx]);
            printf("\n");
        }
    }

    /*........   O - C , nomalized sum square of residuals (R_SUM2)
                Scaled chi square (R_CHI2),
                Scaling const (sigma star = R_SIGMA)
                etc calc.
    */
    nsum=0;
    lt->r_sum2=0.0;
    for(n=1; n<=nobs; n++){
        rsum=0;
        lt->r_omc[n-1]=0.0;
        if(lt->i_good[n-1] > 0){  /*  This is OK obs */
            nsum++;
            for(m=1; m<=lt->n_para; m++){
                rsum+=lt->d_a[n-1+(m-1)*maxy]*lt->x_adj[m-1];
            }
            ominusc=lt->r_y[n-1]-rsum;
            lt->r_omc[n-1]=ominusc;
            lt->r_sum2+=ominusc*ominusc/(lt->r_yerr[n-1]*lt->r_yerr[n-1]);
        }
    }
    lt->n_obs=nsum; /* # of good obs */
    lt->n_free=nsum-lt->m_para; /* this is degree of freedum */

    /* in case of weight having unit  Reduced Chi^2 = Scaleing sigma^2 */
    if(lt->n_free > 0){
        lt->r_chi2=lt->r_sum2/(double)lt->n_free;
        lt->r_sigma=sqrt(lt->r_chi2);
    } else {
        lt->r_sigma=0.0;
        lt->r_chi2=0.0;
    }

    free(d_wa);
    free(du);
    free(dv);
    free(dsigma);
    free(dwork);
    free(d_y);
    free(dfact);
    free(iajst);
}

void dwsvd(int nm, int m, int nn, int n, double *a, double *w,
             int matu, double *u, int matv, double *v,
             int *kerr, double *rv1)
{
/*

      SUBROUTINE DWSVD_2(NM,M,NN,N,A,W,MATU,U,MATV,V,IERR,RV1)
      IMPLICIT REAL*8 (A-H,O-Z)
C     EMA A,W,U,V,RV1
      DIMENSION A(NM,NN),W(NN),U(NM,NN),V(NM,NN),RV1(NN)
      LOGICAL MATU,MATV
C
C   EMA version
C     This subroutine is a copy of SVD, in the
C     "Keisanki no tameno suuchi keisanhou"
C
C     This subroutine determines the singular value decomposition
C        T
C     A=USV of a real M by N rectangular matrix.  Householder
C     bidiagonalization and a variant of the QR algorithm are used.
C
C     On input.
C
C       NM must be set to the row dimension of two-dimensional
C          array parameters as declared in the calling program
C          dimension statement. Note that NM must be at least
C          as large as the maximum of M and N.
C
C       M is the number of rows of A (and U).
C
C       NN is the number of columns of A (and U) and the order of V.
C
C       N is the real number of columns of A (and U) and the order of V.
C
C       A contains the rectangular input matrix to be decomposed.
C
C       MATU should be set to .TRUE. if the U matrix in the
C          decomposition is desired, and to .FALSE. otherwise.
C
C       MATV should be set to .TRUE. if the V matrix in the
C          decomposition is desired, and to .FALSE. otherwise.
C
C     On output.
C       A is unaltered (unless overwritten by U or V).
C
C       W contains the N (non-negative) singular values of A (the
C          diagonal elements of S). They are unordered. If an
C          error exit is made, the singular values should be correct
C          for indices IERR+1,IERR+2,...,N.
C
C       U contains the matrix U (orthogonal column vectors) of the
C          decomposition if MATU has been set to .TRUE. otherwise
C          U is used as a temporary array. U may coincide with A.
C          If an error exit is made, the columns of U corresponding
C          to indices of correct singular valuesw should be correct.
C
C       V contains the matrix V (orthogonal) of the decomposition if
C          MATV has been set to .TRUE. otherwise V is not referenced.
C          V may also coincide with A if U is not needed. If an error
C          exit is made, the columns of V corresponding to indices of
C          correct singular values should be correct.
C
C       IERR is set to
C          zero     for normal return,
C          K        if the K-th singular value has not been determined
C                   after 30 iterations.
C
C       RV1 is a temporary storage array.
C
*/
    int ierr;
    int i,j,l,k,ii,i1,mn,kk,k1,its,ll,l1;
    double g,scale,anorm,f,h,c,s,x,y,z,t;

    ierr=0;
    for(i=0; i<m; i++){
        for(j=0; j<n; j++){
            u[i+j*nm]=a[i+j*nm];
        }
    }
    /* ...... HOUSEHOLDER REDUCTION TO BIDIAGONAL FORM ...... */
    g=0.0;
    scale=0.0;
    anorm=0.0;
    for(i=1; i<=n; i++){
        l=i+1;
        rv1[i-1]=scale*g;
        g=0.0;
        s=0.0;
        scale=0.0;
        if(i > m) goto l210;

        for(k=i; k<=m; k++){
            scale+=fabs(u[(k-1)+(i-1)*nm]);
        }
        if(scale == 0.0) goto l210;

        for(k=i; k<=m; k++){
            u[(k-1)+(i-1)*nm]/=scale;
            s+= u[(k-1)+(i-1)*nm]*u[(k-1)+(i-1)*nm];
        }
        f=u[(i-1)+(i-1)*nm];
        /* G=-DSIGN(DSQRT(S),F)  in FORTRAN */
        if(f >= 0.0){
            g=-sqrt(s);
        } else {
            g=sqrt(s);
        }
        h=f*g-s;
        u[(i-1)+(i-1)*nm]=f-g;
        if(i==n) goto l190;
        for(j=l; j<=n; j++){
            s=0.0;
            for(k=i; k<=m; k++){
                s+=u[(k-1)+(i-1)*nm]*u[(k-1)+(j-1)*nm];
            }
            f=s/h;

            for(k=i; k<=m; k++){
                u[(k-1)+(j-1)*nm]+=f*u[(k-1)+(i-1)*nm];
            }
        }

l190:
        for(k=i; k<=m; k++){
            u[(k-1)+(i-1)*nm]*=scale;
        }

l210:
        w[i-1]=scale*g;
        g=0.0;
        s=0.0;
        scale=0.0;
        if(i > m || i==n) goto l290;
        for(k=l; k<=n; k++){
            scale+=fabs(u[(i-1)+(k-1)*nm]);
        }

        if(scale == 0.0) goto l290;

        for(k=l; k<=n; k++){
            u[(i-1)+(k-1)*nm]/=scale;
            s+=u[(i-1)+(k-1)*nm]*u[(i-1)+(k-1)*nm];
        }

        f=u[(i-1)+(l-1)*nm];
        if(f >= 0.0){
            g=-sqrt(s);
        } else {
            g=sqrt(s);
        }
        h=f*g-s;
        u[(i-1)+(l-1)*nm]=f-g;

        for(k=l; k<=n; k++){
            rv1[k-1]=u[(i-1)+(k-1)*nm]/h;
        }

        if(i==m) goto l270;

        for(j=l; j<=m; j++){
            s=0.0;
            for(k=l; k<=n; k++){
                s+=u[(j-1)+(k-1)*nm]*u[(i-1)+(k-1)*nm];
            }
            for(k=l; k<=n; k++){
                u[(j-1)+(k-1)*nm]+=s*rv1[k-1];
            }
        }
l270:
        for(k=l; k<=n; k++){
            u[(i-1)+(k-1)*nm]*=scale;
        }
l290:
        if(fabs(w[i-1])+fabs(rv1[i-1]) > anorm) anorm=fabs(w[i-1])+fabs(rv1[i-1]);
    }

    /* ..... ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS ....... */
    if(matv != 1) goto l410;
    /*  ..... FOR I=N STEP -1 UNTIL 1 DO -- .......... */

    for(ii=1; ii<=n; ii++){
        i=n+1-ii;
        if(i==n) goto l390;
        if(g == 0.0) goto l360;
        for(j=l; j<=n; j++){
            v[(j-1)+(i-1)*nm]=(u[(i-1)+(j-1)*nm]/u[(i-1)+(l-1)*nm])/g;
        }
        for(j=l; j<=n; j++){
            s=0.0;
            for(k=l; k<=n; k++){
                s+=u[(i-1)+(k-1)*nm]*v[(k-1)+(j-1)*nm];
            }
            for(k=l; k<=n; k++){
                v[(k-1)+(j-1)*nm]+=s*v[(k-1)+(i-1)*nm];
            }
        }
l360:
        for(j=l; j<=n; j++){
            v[(i-1)+(j-1)*nm]=0.0;
            v[(j-1)+(i-1)*nm]=0.0;
        }
l390:
        v[(i-1)+(i-1)*nm]=1.0;
        g=rv1[i-1];
        l=i;
    }
    /*..... ACCUMULATION OF LEFT-HAND TRANSFORMATIONS .....*/
l410:
    if(matu != 1) goto l510;

    /*..... FOR I=MIN(M,N) STEP -1 UNTIL 1 DO -- .........*/
    mn=n;
    if(m < n) mn=m;

    for(ii=1; ii<=mn; ii++){
        i=mn+1-ii;
        l=i+1;
        g=w[i-1];
        if(i==n) goto l430;


        for(j=l; j<=n; j++){
            u[(i-1)+(j-1)*nm]=0.0;
        }
l430:
        if(g == 0.0) goto l475;
        if(i == mn) goto l460;

        for(j=l; j<=n; j++){
            s=0.0;
            for(k=l; k<=m; k++){
                s+=u[(k-1)+(i-1)*nm]*u[(k-1)+(j-1)*nm];
            }
            /*...... DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW .....*/
            f=(s/u[(i-1)+(i-1)*nm])/g;

            for(k=i; k<=m; k++){
                u[(k-1)+(j-1)*nm]+=f*u[(k-1)+(i-1)*nm];
            }
        }
l460:
        for(j=i; j<=m; j++){
            u[(j-1)+(i-1)*nm]/=g;
        }
        goto l490;

l475:
        for(j=i; j<=m; j++){
            u[(j-1)+(i-1)*nm]=0.0;
        }
l490:
        u[(i-1)+(i-1)*nm]+=1.0;
    }
    /*..... DIAGONALIZATION OF THE BIDIAGONAL FORM .........*/
    /*..... FOR K=N STEP -1 UNTIL 1 DO -- ........*/
l510:
    for(kk=1; kk<=n; kk++){
        k1=n-kk;
        k=k1+1;
        its=0;

        /*  debug */
        /*
        //for(j=0;j<m;j++){
        //   for(ll=0;ll<n;ll++){
        //      printf("kk,j,ll,u %d %d %d %f\n",kk,j,ll,u[j+ll*nm]);
        //   }
        //}
        */
        /*   debug end */

        /*..... TEST FOR SPLITTING.  */
        /*      FOR L=K STEP -1 UNTIL 1 DO -- .........*/
l520:

        for(ll=1; ll<=k; ll++){
            l1=k-ll;
            l=l1+1;
            if(fabs(rv1[l-1])+anorm == anorm) goto l565;
            /*... RV1(I) IS ALWAYS ZERO, SO THERE IS NO EXIT
               THROUGH THE BOTTOM OF THE LOOP ......  */
            if(fabs(w[l1-1])+anorm == anorm) goto l540;
        }
        /*..... CANCELLATION OF RV1(L) IF L GREATER THAN 1 ....*/
l540:
        c=0.0;
        s=1.0;

        for(i=l; i<=k; i++){
            f=s*rv1[i-1];
            rv1[i-1]*=c;
            if(fabs(f)+anorm == anorm) goto l565;
            g=w[i-1];
            h=sqrt(f*f+g*g);
            w[i-1]=h;
            c=g/h;
            s=-f/h;
            if(matu != 1) goto l560;

            for(j=1; j<=m; j++){
                y=u[(j-1)+(l1-1)*nm];
                z=u[(j-1)+(i-1)*nm];
                u[(j-1)+(l1-1)*nm]=y*c+z*s;
                u[(j-1)+(i-1)*nm]=-y*s+z*c;
            }
l560:;
        }

        /*..... TEST FOR CONVERGENCE ......*/
l565:
        z=w[k-1];
        if(l==k) goto l650;
        /*.... SHIFT FROM BOTTOM 2 BY 2 MINOR ....*/
        if(its==30) goto l1000;
        its++;
        x=w[l-1];
        y=w[k1-1];
        g=rv1[k1-1];
        h=rv1[k-1];
        f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
        g=sqrt(f*f+1.0);
        if(f >= 0.0){
            t=g;
        } else {
            t=-g;
        }
        f=((x-z)*(x+z)+h*(y/(f+t)-h))/x;
        /*.... NEXT QR TRANSFORMATION ....*/
        c=1.0;
        s=1.0;

        for(i1=l; i1<=k1; i1++){
            i=i1+1;
            g=rv1[i-1];
            y=w[i-1];
            h=s*g;
            g=g*c;
            z=sqrt(f*f+h*h);
            rv1[i1-1]=z;
            c=f/z;
            s=h/z;
            f=x*c+g*s;
            g=-x*s+g*c;
            h=y*s;
            y=y*c;
            if(matv != 1) goto l575;

            for(j=1; j<=n; j++){
                x=v[(j-1)+(i1-1)*nm];
                z=v[(j-1)+(i-1)*nm];
                v[(j-1)+(i1-1)*nm]=x*c+z*s;
                v[(j-1)+(i-1)*nm]=-x*s+z*c;
            }
l575:
            z=sqrt(f*f+h*h);
            w[i1-1]=z;

            /*... ROTATION CAN BE ARBITRARY IF Z IS ZERO ....*/
            if(z==0.0) goto l580;
            c=f/z;
            s=h/z;
l580:
            f=c*g+s*y;
            x=-s*g+c*y;
            if(matu != 1) goto l600;
            for(j=1; j<=m; j++){
                y=u[(j-1)+(i1-1)*nm];
                z=u[(j-1)+(i-1)*nm];
                u[(j-1)+(i1-1)*nm]=y*c+z*s;
                u[(j-1)+(i-1)*nm]=-y*s+z*c;
            }
l600:;
        }

        rv1[l-1]=0.0;
        rv1[k-1]=f;
        w[k-1]=x;
        goto l520;
        /*.... CONVERGENCE ......*/
l650:
        if(z>=0.0) goto l700;
        /*... W(K) IS MADE NON-NEGATIVE ......*/
        w[k-1]=-z;
        if(matv != 1) goto l700;

        for(j=1; j<=n; j++) v[(j-1)+(k-1)*nm]*=-1.0;
l700:;
     }

     goto l1001;

     /*...... SET ERROR -- NO CONVERGENCE TO A
              SINGULAR VALUE AFTER 30 ITERATIONS ....*/
l1000:
     ierr=k;
l1001:
     *kerr=ierr;
}

