/**************************************************************/
/*                                                            */
/*   File name :  liblsq.h                                    */
/*                                                            */
/*      Version:   2005-05-12                                 */
/*                                                            */
/*   Copyright (c) 2005 T.Kondo/NICT(CRL) All Right Reserved  */
/*                                                            */
/**************************************************************/
/*

Ver: 2005-05-12
     Start migration from Fortran liblsq library

*/

#ifndef _LIBLSQ_
#define _LIBLSQ_

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif


typedef struct lsq_type /* file name expression for komb */
{

    double *r_y;    /* Observed data (O-C in case of non linear lsq) [2500]*/
    double *r_yerr; /* 1 sigma error of observed data [2500]*/
    int *i_good;    /* Flag indicate good obs data   1:OK  0: NO [2500]*/
    double *d_a;    /* Jacobian (original) not affected [2500X100]*/
    double *r_omc;  /* O-C after fitting  [2500]*/
    double *x_adj;  /* Adjustment value calculated   [100]        */
                    /*       must be added to paramters in main   */
    int *i_flag;    /*  Adjusting parameter indicator    [100]         */
                    /*    1: Adjust  0 : no Adjust  will be affected   */
                    /*    -1 means rank ochi occurred parameter        */
    double *d_corx; /* Obtained correlation matrix [100X100]*/
    double *d_covx; /* Obtained covariance matrix  [100X100]*/
    double r_sum2;  /* Sum of residual square   */
    double r_sigma; /* Residual 1 sigma         */
    double r_chi2;  /* Reduced chi squer        */
    int n_free;     /* Degree of freedum */
    int m_pmax;     /* Available max # of parameters ex. =100 */
    int m_omax;     /*    "               data ex. =2500 */
    int n_data;     /* # of data valid   <= m_omax       */
    int n_obs;      /* Actual # of data use for estimation */
    int n_para;     /* # of parameters in X_ADJ etc.  */
    int m_para;     /* Actual # of parameters fitted  */
    int i_dbk;      /* Debug control   1: ON   0: OFF */
    int ierr;       /* to get error code from each function call */

}lsq_type_t;
void lsqinit(lsq_type_t *lt, int maxobs, int maxpara);
void lsqfinish(lsq_type_t *lt);
void lsqdebug(lsq_type_t *lt);
void lsqgetmax(lsq_type_t *lt, int *maxobs, int *maxpara);
void lsqsetsize(lsq_type_t *lt, int nobs, int npara);
void lsqgetsize(lsq_type_t *lt, int *nobs, int *npara);
void lsqsetflag(lsq_type_t *lt, char *a1, int nm, char *a2);
void lsqgetflag(lsq_type_t *lt, char *a1, int nm, int *iflag);
void lsqpflag(lsq_type_t *lt, int m);
void lsqoflag(lsq_type_t *lt, int n);
void lsqoflagr(lsq_type_t *lt, int n);
void lsqpflagr(lsq_type_t *lt, int m);
void lsqgetpflag(lsq_type_t *lt, int m, int *iflag);
void lsqgetoflag(lsq_type_t *lt, int n, int *igood);
void lsqsetjacob(lsq_type_t *lt, double a, int n, int m);
void lsqgetjacob(lsq_type_t *lt, double *a, int n, int m);
void lsqsetobs(lsq_type_t *lt, double y, double yerr, int n);
void lsqgetobs(lsq_type_t *lt, double *y, double *yerr, int n);
void lsqgetkai(lsq_type_t *lt, double *x, double *xerr, int m);
void lsqgetcor(lsq_type_t *lt, double *c, int m1, int m2);
void lsqgetcov(lsq_type_t *lt, double *v, int m1, int m2);
void lsqgetomc(lsq_type_t *lt, double *omc, int n);
void lsqgetstd(lsq_type_t *lt, double *rsum2, double *sigma,
               double *rchi2, int *nfree, int *ndata, int *mpara);
void dolsq(lsq_type_t *lt);
void dwsvd(int nm, int m, int nn, int n, double *a, double *w,
             int matu, double *u, int matv, double *v,
             int *kerr, double *rv1);

#endif

